@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2011-2020 Sutou Kouhei <kou@clear-code.com>
-  Copyright(C) 2020 Horimoto Yasuhiro <horimoto@clear-code.com>
+  Copyright(C) 2020-2021 Horimoto Yasuhiro <horimoto@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -59,10 +59,6 @@
   typedef char *range_id_t;
 #endif
 
-#if defined(MRN_MARIADB_P) || MYSQL_VERSION_ID < 80002
-  typedef st_select_lex SELECT_LEX;
-#endif
-
 #if MYSQL_VERSION_ID >= 50607
 #  define MRN_SUPPORT_FOREIGN_KEYS 1
 #endif
@@ -419,27 +415,51 @@ typedef HASH mrn_table_def_cache_type;
 #endif
 
 #if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
-#  define MRN_SELECT_LEX_GET_WHERE_COND(select_lex) \
-  ((select_lex)->where_cond())
-#  define MRN_SELECT_LEX_GET_HAVING_COND(select_lex) \
-  ((select_lex)->having_cond())
-#  define MRN_SELECT_LEX_GET_ACTIVE_OPTIONS(select_lex) \
-  ((select_lex)->active_options())
+#  define MRN_QUERY_BLOCK_GET_WHERE_COND(query_block) \
+  ((query_block)->where_cond())
+#  define MRN_QUERY_BLOCK_GET_HAVING_COND(query_block) \
+  ((query_block)->having_cond())
+#  define MRN_QUERY_BLOCK_GET_ACTIVE_OPTIONS(query_block) \
+  ((query_block)->active_options())
 #else
-#  define MRN_SELECT_LEX_GET_WHERE_COND(select_lex) \
-  ((select_lex)->where)
-#  define MRN_SELECT_LEX_GET_HAVING_COND(select_lex) \
-  ((select_lex)->having)
-#  define MRN_SELECT_LEX_GET_ACTIVE_OPTIONS(select_lex) \
-  ((select_lex)->options)
+#  define MRN_QUERY_BLOCK_GET_WHERE_COND(query_block) \
+  ((query_block)->where)
+#  define MRN_QUERY_BLOCK_GET_HAVING_COND(query_block) \
+  ((query_block)->having)
+#  define MRN_QUERY_BLOCK_GET_ACTIVE_OPTIONS(query_block) \
+  ((query_block)->options)
 #endif
 
 #if MYSQL_VERSION_ID >= 80021 && !defined(MRN_MARIADB_P)
-#  define MRN_SELECT_LEX_GET_FIELDS_LIST(select_lex) \
-  ((select_lex)->fields_list)
+#  define MRN_QUERY_BLOCK_GET_FIELDS_LIST(query_block) \
+  ((query_block)->fields_list)
 #else
-#  define MRN_SELECT_LEX_GET_FIELDS_LIST(select_lex) \
-  ((select_lex)->item_list)
+#  define MRN_QUERY_BLOCK_GET_FIELDS_LIST(query_block) \
+  ((query_block)->item_list)
+#endif
+
+#if MYSQL_VERSION_ID >= 80022 && !defined(MRN_MARIADB_P)
+#  define MRN_QUERY_BLOCK_GET_NUM_VISIBLE_FIELDS(query_block) \
+  ((unsigned int)(query_block->num_visible_fields()))
+#else
+#  define MRN_QUERY_BLOCK_GET_NUM_VISIBLE_FIELDS(query_block) \
+  (MRN_QUERY_BLOCK_GET_FIELDS_LIST(query_block).elements)
+#endif
+
+#if MYSQL_VERSION_ID >= 80022 && !defined(MRN_MARIADB_P)
+#  define MRN_QUERY_BLOCK_GET_FIRST_VISIBLE_FIELD(query_block) \
+  (*((query_block)->visible_fields().begin()))
+#else
+#  define MRN_QUERY_BLOCK_GET_FIRST_VISIBLE_FIELD(query_block) \
+  (static_cast<Item *>(MRN_QUERY_BLOCK_GET_FIELDS_LIST(query_block).first_node()->info))
+#endif
+
+#if MYSQL_VERSION_ID >= 80022 && !defined(MRN_MARIADB_P)
+#  define MRN_QUERY_BLOCK_HAS_LIMIT(query_block) \
+  (query_block->has_limit())
+#else
+#  define MRN_QUERY_BLOCK_HAS_LIMIT(query_block) \
+  (query_block->explicit_limit)
 #endif
 
 #if defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100000
@@ -578,7 +598,11 @@ typedef HASH mrn_table_def_cache_type;
   (create_subpartition_name(out, in1, in2, in3, name_variant), 0)
 #endif
 
-#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80024)
+#  define ITEM_SUM_GET_NEST_LEVEL(sum_item) (sum_item)->base_query_block->nest_level
+#  define ITEM_SUM_GET_AGGR_LEVEL(sum_item) (sum_item)->aggr_query_block->nest_level
+#  define ITEM_SUM_GET_MAX_AGGR_LEVEL(sum_item) (sum_item)->max_aggr_level
+#elif (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
 #  define ITEM_SUM_GET_NEST_LEVEL(sum_item) (sum_item)->base_select->nest_level
 #  define ITEM_SUM_GET_AGGR_LEVEL(sum_item) (sum_item)->aggr_select->nest_level
 #  define ITEM_SUM_GET_MAX_AGGR_LEVEL(sum_item) (sum_item)->max_aggr_level
@@ -666,6 +690,14 @@ typedef HASH mrn_table_def_cache_type;
   (table_list)->table_name_length
 #endif
 
+#if !defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80024
+#  define MRN_TABLE_LIST_QUERY_BLOCK(table_list)    \
+   (table_list)->query_block
+#else
+#  define MRN_TABLE_LIST_QUERY_BLOCK(table_list)    \
+   (table_list)->select_lex
+#endif
+
 #if !defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80018
 #  define MRN_DECLARE_TABLE_LIST(variable_name,                         \
                                  db_name,                               \
@@ -761,11 +793,28 @@ typedef HASH mrn_table_def_cache_type;
   typedef Foreign_key mrn_foreign_key_spec;
 #endif
 
-#if MYSQL_VERSION_ID >= 80011 && !defined(MRN_MARIADB_P)
+#if MYSQL_VERSION_ID >= 80024 && !defined(MRN_MARIADB_P)
+  class Query_block;
+  typedef Query_block mrn_query_block;
+#elif MYSQL_VERSION_ID >= 80011 && !defined(MRN_MARIADB_P)
   class SELECT_LEX;
-  typedef SELECT_LEX mrn_select_lex;
+  typedef SELECT_LEX mrn_query_block;
 #else
-  typedef st_select_lex mrn_select_lex;
+  typedef st_select_lex mrn_query_block;
+#endif
+
+#if MYSQL_VERSION_ID >= 80024 && !defined(MRN_MARIADB_P)
+  typedef Query_expression mrn_query_expression;
+#else
+  typedef SELECT_LEX_UNIT mrn_query_expression;
+#endif
+
+#if MYSQL_VERSION_ID >= 80024 && !defined(MRN_MARIADB_P)
+#  define MRN_QUERY_EXPRESSION_FIRST_QUERY_BLOCK(query_expression)  \
+   (query_expression)->first_query_block()
+#else
+#  define MRN_QUERY_EXPRESSION_FIRST_QUERY_BLOCK(query_expression)  \
+   (query_expression)->first_select()
 #endif
 
 #if MYSQL_VERSION_ID >= 80011 && !defined(MRN_MARIADB_P)
@@ -924,3 +973,10 @@ typedef HASH mrn_table_def_cache_type;
 #  define MRN_BITMAP_INIT(map, buf, n_bits) \
     bitmap_init((map), (buf), (n_bits), false)
 #endif
+
+#if defined(MRN_MARIADB_P) && ((MYSQL_VERSION_ID >= 100237 && MYSQL_VERSION_ID < 100300) || \
+                               (MYSQL_VERSION_ID >= 100328 && MYSQL_VERSION_ID < 100400) || \
+                               (MYSQL_VERSION_ID >= 100418 && MYSQL_VERSION_ID < 100500) || \
+                               (MYSQL_VERSION_ID >= 100509))
+#  define MRN_DBUG_TMP_USE_BITMAP_PP
+#endif
