@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2; indent-tabs-mode: nil -*- */
 /*
-  Copyright(C) 2015-2019 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2015-2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -29,6 +29,8 @@
 #include <mrn_variables.hpp>
 #include <mrn_query_parser.hpp>
 #include <mrn_current_thread.hpp>
+#include <mrn_index_table_name.hpp>
+#include <mrn_smart_grn_obj.hpp>
 
 MRN_BEGIN_DECLS
 
@@ -38,15 +40,14 @@ extern mrn::ContextPool *mrn_context_pool;
 typedef struct st_mrn_snippet_html_info
 {
   grn_ctx *ctx;
+  grn_obj result;
+  unsigned int mysql_table_name_index;
+  unsigned int mysql_index_name_index;
   grn_obj *db;
   bool use_shared_db;
   grn_obj *snippet;
-  grn_obj result;
-  struct {
-    bool used;
-    grn_obj *table;
-    grn_obj *default_column;
-  } query_mode;
+  grn_obj *query_table;
+  grn_obj *query_default_column;
 } mrn_snippet_html_info;
 
 static mrn_bool mrn_snippet_html_prepare(mrn_snippet_html_info *info,
@@ -63,14 +64,10 @@ static mrn_bool mrn_snippet_html_prepare(mrn_snippet_html_info *info,
   const char *open_tag = "<span class=\"keyword\">";
   const char *close_tag = "</span>";
   grn_snip_mapping *mapping = GRN_SNIP_MAPPING_HTML_ESCAPE;
-  grn_obj *expr = NULL;
 
   *snippet = NULL;
 
   mrn::encoding::set_raw(ctx, system_charset_info);
-  if (!(system_charset_info->state & (MY_CS_BINSORT | MY_CS_CSSORT))) {
-    flags |= GRN_SNIP_NORMALIZE;
-  }
 
   *snippet = grn_snip_open(ctx, flags,
                            width, max_n_results,
@@ -86,74 +83,133 @@ static mrn_bool mrn_snippet_html_prepare(mrn_snippet_html_info *info,
     goto error;
   }
 
-  if (info->query_mode.used) {
-    if (!info->query_mode.table) {
-      grn_obj *short_text;
-      short_text = grn_ctx_at(info->ctx, GRN_DB_SHORT_TEXT);
-      info->query_mode.table = grn_table_create(info->ctx,
-                                                NULL, 0, NULL,
-                                                GRN_TABLE_HASH_KEY,
-                                                short_text,
-                                                NULL);
-    }
-    if (!info->query_mode.default_column) {
-      info->query_mode.default_column =
-        grn_obj_column(info->ctx,
-                       info->query_mode.table,
-                       GRN_COLUMN_NAME_KEY,
-                       GRN_COLUMN_NAME_KEY_LEN);
-    }
-
-    grn_obj *record = NULL;
-    GRN_EXPR_CREATE_FOR_QUERY(info->ctx, info->query_mode.table, expr, record);
-    if (!expr) {
-      if (message) {
-        snprintf(message, MYSQL_ERRMSG_SIZE,
-                 "mroonga_snippet_html(): "
-                 "failed to create expression: <%s>",
-                 ctx->errbuf);
-      }
-      goto error;
-    }
-
-    mrn::QueryParser query_parser(info->ctx,
-                                  current_thd,
-                                  expr,
-                                  info->query_mode.default_column,
-                                  0,
-                                  NULL);
-    grn_rc rc = query_parser.parse(args->args[1], args->lengths[1]);
-    if (rc != GRN_SUCCESS) {
-      if (message) {
-        snprintf(message, MYSQL_ERRMSG_SIZE,
-                 "mroonga_snippet_html(): "
-                 "failed to parse query: <%s>",
-                 ctx->errbuf);
-      }
-      goto error;
-    }
-
-    rc = grn_expr_snip_add_conditions(info->ctx,
-                                      expr,
-                                      *snippet,
-                                      0,
-                                      NULL, NULL,
-                                      NULL, NULL);
-    if (rc != GRN_SUCCESS) {
-      if (message) {
-        snprintf(message, MYSQL_ERRMSG_SIZE,
-                 "mroonga_snippet_html(): "
-                 "failed to add conditions: <%s>",
-                 ctx->errbuf);
-      }
-      goto error;
+  if (info->mysql_table_name_index == 0 &&
+      info->mysql_index_name_index == 0) {
+    if (!(system_charset_info->state & (MY_CS_BINSORT | MY_CS_CSSORT))) {
+      grn_snip_set_normalizer(ctx, *snippet, GRN_NORMALIZER_AUTO);
     }
   } else {
-    unsigned int i;
-    for (i = 1; i < args->arg_count; ++i) {
-      if (!args->args[i]) {
-        continue;
+    const char *mysql_table_name = args->args[info->mysql_table_name_index];
+    const size_t mysql_table_name_length =
+      args->lengths[info->mysql_table_name_index];
+    if (mysql_table_name_length == 0) {
+      if (message) {
+        snprintf(message, MYSQL_ERRMSG_SIZE,
+                 "mroonga_snippet_html(): table_name is missing");
       }
+      goto error;
+    }
+    const char *mysql_index_name = args->args[info->mysql_index_name_index];
+    const size_t mysql_index_name_length =
+      args->lengths[info->mysql_index_name_index];
+    if (mysql_index_name_length == 0) {
+      if (message) {
+        snprintf(message, MYSQL_ERRMSG_SIZE,
+                 "mroonga_snippet_html(): index_name is missing");
+      }
+      goto error;
+    }
+    mrn::IndexTableName index_table_name(mysql_table_name,
+                                         mysql_table_name_length,
+                                         mysql_index_name,
+                                         mysql_index_name_length);
+    mrn::SmartGrnObj lexicon(ctx,
+                             index_table_name.c_str(),
+                             index_table_name.length());
+    if (!lexicon.get()) {
+      if (message) {
+        snprintf(message, MYSQL_ERRMSG_SIZE,
+                 "mroonga_snippet_html(): nonexistent index: <%.*s.%.*s>",
+                 static_cast<int>(mysql_table_name_length),
+                 mysql_table_name,
+                 static_cast<int>(mysql_index_name_length),
+                 mysql_index_name);
+      }
+      goto error;
+    }
+    grn_snip_set_normalizer(ctx, *snippet, lexicon.get());
+  }
+
+  for (unsigned int i = 1; i < args->arg_count; ++i) {
+    if (!args->args[i]) {
+      continue;
+    }
+    grn_raw_string arg_name = {
+      args->attributes[i],
+      args->attribute_lengths[i],
+    };
+    if (GRN_RAW_STRING_EQUAL_CSTRING(arg_name, "table_name") ||
+        GRN_RAW_STRING_EQUAL_CSTRING(arg_name, "index_name")) {
+      // Do nothing
+    } else if (GRN_RAW_STRING_EQUAL_CSTRING(arg_name, "query")) {
+      if (!info->query_table) {
+        grn_obj *short_text;
+        short_text = grn_ctx_at(info->ctx, GRN_DB_SHORT_TEXT);
+        info->query_table = grn_table_create(info->ctx,
+                                             NULL, 0, NULL,
+                                             GRN_TABLE_HASH_KEY,
+                                             short_text,
+                                             NULL);
+      }
+      if (!info->query_default_column) {
+        info->query_default_column =
+          grn_obj_column(info->ctx,
+                         info->query_table,
+                         GRN_COLUMN_NAME_KEY,
+                         GRN_COLUMN_NAME_KEY_LEN);
+      }
+
+      grn_obj *expr = NULL;
+      grn_obj *record = NULL;
+      GRN_EXPR_CREATE_FOR_QUERY(info->ctx, info->query_table, expr, record);
+      if (!expr) {
+        if (message) {
+          snprintf(message, MYSQL_ERRMSG_SIZE,
+                   "mroonga_snippet_html(): "
+                   "failed to create expression: <%s>",
+                   ctx->errbuf);
+        }
+        goto error;
+      }
+
+      mrn::SmartGrnObj smart_expr(ctx, expr);
+      mrn::QueryParser query_parser(info->ctx,
+                                    current_thd,
+                                    expr,
+                                    info->query_default_column,
+                                    0,
+                                    NULL);
+      grn_rc rc = query_parser.parse(args->args[i], args->lengths[i]);
+      if (rc != GRN_SUCCESS) {
+        if (message) {
+          snprintf(message, MYSQL_ERRMSG_SIZE,
+                   "mroonga_snippet_html(): "
+                   "failed to parse query: "
+                   "<%.*s>: "
+                   "<%s>",
+                   static_cast<int>(args->lengths[i]),
+                   args->args[i],
+                   ctx->errbuf);
+        }
+        goto error;
+      }
+
+      rc = grn_expr_snip_add_conditions(info->ctx,
+                                        expr,
+                                        *snippet,
+                                        0,
+                                        NULL, NULL,
+                                        NULL, NULL);
+      if (rc != GRN_SUCCESS) {
+        if (message) {
+          snprintf(message, MYSQL_ERRMSG_SIZE,
+                   "mroonga_snippet_html(): "
+                   "failed to add conditions: <%s>",
+                   ctx->errbuf);
+        }
+        goto error;
+      }
+    } else {
       grn_rc rc = grn_snip_add_cond(ctx, *snippet,
                                     args->args[i], args->lengths[i],
                                     NULL, 0,
@@ -173,9 +229,6 @@ static mrn_bool mrn_snippet_html_prepare(mrn_snippet_html_info *info,
   DBUG_RETURN(false);
 
 error:
-  if (expr) {
-    grn_obj_close(ctx, expr);
-  }
   if (*snippet) {
     grn_obj_close(ctx, *snippet);
   }
@@ -183,8 +236,8 @@ error:
 }
 
 MRN_API mrn_bool mroonga_snippet_html_init(UDF_INIT *init,
-                                          UDF_ARGS *args,
-                                          char *message)
+                                           UDF_ARGS *args,
+                                           char *message)
 {
   MRN_DBUG_ENTER_FUNCTION();
 
@@ -264,27 +317,59 @@ MRN_API mrn_bool mroonga_snippet_html_init(UDF_INIT *init,
       goto error;
     }
   }
+  GRN_TEXT_INIT(&(info->result), 0);
+  info->mysql_table_name_index = 0;
+  info->mysql_index_name_index = 0;
 
-  info->query_mode.used = false;
-
-  if (args->arg_count == 2 &&
-      args->attribute_lengths[1] == strlen("query") &&
-      strncmp(args->attributes[1], "query", strlen("query")) == 0) {
-    info->query_mode.used = true;
-    info->query_mode.table = NULL;
-    info->query_mode.default_column = NULL;
+  info->query_table = NULL;
+  info->query_default_column = NULL;
+  for (unsigned int i = 1; i < args->arg_count; ++i) {
+    grn_raw_string arg_name = {
+      args->attributes[i],
+      args->attribute_lengths[i],
+    };
+    if (GRN_RAW_STRING_EQUAL_CSTRING(arg_name, "query") ||
+        GRN_RAW_STRING_EQUAL_CSTRING(arg_name, "keyword")) {
+      // Do nothing
+    } else if (GRN_RAW_STRING_EQUAL_CSTRING(arg_name, "table_name")) {
+      if (info->mysql_table_name_index != 0) {
+        sprintf(message,
+                "mroonga_snippet_html(): "
+                "can't specify table_name multiple times");
+        goto error;
+      }
+      info->mysql_table_name_index = i;
+    } else if (GRN_RAW_STRING_EQUAL_CSTRING(arg_name, "index_name")) {
+      if (info->mysql_index_name_index != 0) {
+        sprintf(message,
+                "mroonga_snippet_html(): "
+                "can't specify index_name multiple times");
+        goto error;
+      }
+      info->mysql_index_name_index = i;
+    } else {
+      if (arg_name.length > 0 && arg_name.value[0] == '\'') {
+        // No "AS XXX"
+      } else {
+        sprintf(message,
+                "mroonga_snippet_html(): unknown named argument: %.*s",
+                static_cast<int>(arg_name.length),
+                arg_name.value);
+        goto error;
+      }
+    }
   }
 
   {
-    bool all_keywords_are_constant = true;
+    bool all_arguments_are_constant = true;
     for (unsigned int i = 1; i < args->arg_count; ++i) {
       if (!args->args[i]) {
-        all_keywords_are_constant = false;
+        all_arguments_are_constant = false;
         break;
       }
     }
 
-    if (all_keywords_are_constant) {
+    if (all_arguments_are_constant) {
       if (mrn_snippet_html_prepare(info, args, message, &(info->snippet))) {
         goto error;
       }
@@ -299,6 +384,9 @@ MRN_API mrn_bool mroonga_snippet_html_init(UDF_INIT *init,
 
 error:
   if (info) {
+    if (info->db) {
+      GRN_OBJ_FIN(info->ctx, &(info->result));
+    }
     if (!info->use_shared_db) {
       grn_obj_close(info->ctx, info->db);
     }
@@ -415,13 +503,11 @@ MRN_API void mroonga_snippet_html_deinit(UDF_INIT *init)
   if (info->snippet) {
     grn_obj_close(info->ctx, info->snippet);
   }
-  if (info->query_mode.used) {
-    if (info->query_mode.default_column) {
-      grn_obj_close(info->ctx, info->query_mode.default_column);
-    }
-    if (info->query_mode.table) {
-      grn_obj_close(info->ctx, info->query_mode.table);
-    }
+  if (info->query_default_column) {
+    grn_obj_close(info->ctx, info->query_default_column);
+  }
+  if (info->query_table) {
+    grn_obj_close(info->ctx, info->query_table);
   }
   GRN_OBJ_FIN(info->ctx, &(info->result));
   if (!info->use_shared_db) {
